@@ -17,88 +17,6 @@ function getContacts() {
     return [{ folder_id: activefolder, id: contactDetailId }];
 }
 
-// The menu
-
-register("OX_SELECTED_ITEMS_CHANGED", updateMenu);
-register("OX_menu_Change_Height", updateMenu);
-register("OX_Switched", updateMenu);
-menuarrows["com.4psa.voipnow"] = {};
-var menu = MenuNodes.createSmallButtonContext("com.4psa.voipnow.menu",
-    _("VoipNow"), "com.4psa.voipnow");
-
-/**
- * A menu section which selects one of many entries.
- * @param {String} module A unique module name for internal use.
- * @param {I18nString} caption The caption of the menu section.
- * @param {Array of I18nString} names An array with the section's entries.
- * @param {Array} values An array with one value for each entry in the #names
- * array. Must have the same length as #names.
- * @type String
- * @return The unique ID of the section, which is required by changeDisplay etc.
- */
-function MenuSelection(module, caption, names, values) {
-    this.module = module;
-    this.count = names.length;
-    this.id = module + ".menuselection" + MenuSelection.id++;
-    this.index = 0;
-    var menu = MenuNodes.createSmallButtonContext(this.id, caption, module);
-    menu.body.parentNode.parentNode.className = "backgroundheight";
-    if (!(module in menuarrows)) menuarrows[module] = {};
-    var Self = this;
-    for (var i = 0; i < this.count; i++) {
-        var id2 = this.id + "." + i;
-        var element = newnode("tr", i ? { display: "none" } : 0,
-            { id: id2 + "_active" },
-            [newnode("td", 0, { className: "height16 background-color-PMG-selection-elements padding7px" },
-                [newnode("div", 0, { className: "height16" },
-                    [addTranslated(names[i])])])]);
-        allnodes[element.id] = element;
-        menu.body.appendChild(element);
-        var element2 = newnode("tr", i ? 0 : { display: "none" },
-            { id: id2 + "_notactive" },
-            [newnode("td", 0, { className: "height16 padding7px" },
-                [newnode("div", 0, { className: "height16" },
-                    [addTranslated(names[i])])])]);
-        allnodes[element2.id] = element2;
-        element2.onclick = (function(index, value) {
-            return function() {
-                Self.set(index);
-                Self.onChange(value);
-            };
-        })(i, values[i]);
-        menu.body.appendChild(element2);
-    }
-    addMenuNode(menu.node, MenuNodes.FIXED, 30);
-    MenuSelection.instances[this.id] = this;
-}
-MenuSelection.id = 0;
-MenuSelection.instances = {};
-MenuSelection.update = function() {
-    for (var i in MenuSelection.instances) MenuSelection.instances[i].set();
-};
-register("OX_SELECTED_ITEMS_CHANGED", MenuSelection.update);
-register("OX_menu_Change_Height", MenuSelection.update);
-register("OX_Switched", MenuSelection.update);
-MenuSelection.prototype.set = function(index) {
-    if (index !== undefined) this.index = index;
-    menuglobalzaehler = 0;
-    menuarrows[this.module][this.id] = [];
-    for (var i = 0; i < this.count; i++) {
-        menu_display_views(this.module, this.id, true, i == this.index,
-                           this.id + "." + i);
-    }
-};
-MenuSelection.prototype.onChange = emptyFunction;
-
-
-function ManualPulldownMenu(trigger) {
-    PopupMenu.call(this);
-    this.trigger = trigger;
-    var Self = this;
-    trigger.oxPulldownTrigger = true;
-}
-ManualPulldownMenu.prototype = extend(PulldownMenu, {});
-
 // Calling
 var phoneFields = ["telephone_ip", "telephone_business1", "telephone_business2",
                    "telephone_callback", "telephone_car", "telephone_company",
@@ -306,7 +224,7 @@ VoipCallDialog.prototype.paintNumbers = function(contact) {
 		}
 		if (contact.unnamedNumbers) {
 		    for (var i = 0; i < contact.unnamedNumbers.length; i++) {
-		        addNumber("", contact.unnamedNumbers[i]);
+		        addNumber(noI18n(""), contact.unnamedNumbers[i]);
 		    }
 		}
 	}
@@ -378,6 +296,7 @@ VoipCallDialog.prototype.paintButtons = function() {
 	var self = this;
 	// close button
 	var button = newnode("input", 0, { type: "button", value: _("Close") });
+    register("LanguageChanged", function() { button.value = _("Close"); });
 	// add listener
 	addDOMEvent(button, "click", function() {
 		self.hide();
@@ -479,209 +398,278 @@ register("PAINT_CONTACT", function(e) {
 
 // ---------------------------------------------
 
-MenuNodes.createSmallButton(menu, "com.4psa.voipnow.call", _("Call"),
-    urlify("plugins/com.4psa.voipnow/images/call.png"),
-    urlify("plugins/com.4psa.voipnow/images/call_d.png"),
-    function() {
-        if (PopupMenu.last == callMenu) {
-            PopupMenu.last = null;
-            return;
-        }
-        var collection = { objects: getContacts(), columns: phoneFields };
-        OXCache.newRequest(null, "contacts", collection, null,
-            function(data) {
-                var contact = data.objects[0];
-                var uid = contact.internal_userid;
-                if (uid) {
-                    ox.JSON.put(AjaxRoot + "/user?action=list&columns=1" + 
-                        "&com.4psa.voipnow=mainExtension&session=" + session,
-                        [contact.internal_userid],
-                        function(data) {
-                            var ext = data.data[0][1];
-                            ext = ext && /^(?:\d+=)?(.*)$/.exec(ext)[1];
-                            if (ext) newcall(ext); else popup();
-                        });
-                } else {
-                    popup();
-                }
-                function popup() {
-                	callDialog.show(contact);
-                }
-                function newcall(number) {
-                    active.wait(function(active) {
-                        ox.JSON.put(AjaxRoot +
-                            "/com.4psa.voipnow?action=newcall&phone=" +
-                            encodeURIComponent(number.replace(/[^0-9#*+]/g, "")) +
-                            "&callerid=" +
-                            encodeURIComponent(contact.display_name) +
-                            "&session=" + session, getPhones(), emptyFunction);
-                    });
-                }
-            });
+// The menu
+
+var callHistoryButtons = [];
+
+function createGlobalTab() {
+    var section = new ox.gui.PanelMenuSection(_("VoipNow"));
+    
+    var callButton = new ox.gui.MenuItem({
+        title: _("Call"),
+        icons: ["plugins/com.4psa.voipnow/images/call.png",
+                "plugins/com.4psa.voipnow/images/call_d.png"],
+        pathPrefix: "",
+        action: callButtonAction
     });
-var callMenu = new ManualPulldownMenu(allnodes["com.4psa.voipnow.call"]);
-var customPhoneInput = newnode("input");
-var customPhone = {
-    node: newnode("div", 0, { className: "PopupEntryDiv" }, [customPhoneInput]),
-    show: emptyFunction,
-    hide: emptyFunction,
-    siblingSelected: emptyFunction
-};
-addDOMEvent(customPhone.node, "mouseover",
-    function() { customPhone.parent.itemSelected(); });
-addDOMEvent(customPhoneInput, "keypress", function(e) {
-    if ((e.keyCode || e.which) == 13) {
-        customPhone.newcall(customPhoneInput.value);
-        customPhone.parent.close();
+    register("OX_SELECTED_ITEMS_CHANGED", updateMenu);
+    register("OX_Switched", updateMenu);
+    function updateMenu() {
+        callButton.setVisible(activemodule == "contacts");
+        callButton.setEnabled(menucountselected == 1);
     }
-});
+    section.add(callButton);
+    
+    section.add(new ox.gui.MenuItem({
+        title: _("Fax"),
+        icons: ["plugins/com.4psa.voipnow/images/fax.png",
+                "plugins/com.4psa.voipnow/images/fax_d.png"],
+        pathPrefix: "",
+        action: faxButtonAction
+    }));
+    
+    var callHistoryButton = new ox.gui.MenuItem({
+        title: _("Call history"),
+        icons: ["img/dummy.gif", "img/dummy.gif"],
+        action: function() {
+            triggerEvent("OX_Switch_View", "com.4psa.voipnow.callhistory");
+            return true;
+        }
+    });
+    callHistoryButtons.push(callHistoryButton);
+    section.add(callHistoryButton);
+    
+    var locationMenu = new ox.gui.MenuItem({
+        title: _("My location"),
+        action: function() {
+            locations.wait(function(locations) {
+                locationMenu.removeChildren();
+                for (var i in locations) {
+                    locationMenu.add(new ox.gui.MenuItem({
+                        title: noI18n(locations[i].name),
+                        action: (function(id) { return function() {
+                            active.update(id);
+                            return true;
+                        }; })(i)
+                    }));
+                }
+                locationMenu.validate();
+            });
+        }
+    });
+    section.add(locationMenu);
+    
+    locations.register(function(locations, join) {
+        locationMenu.removeChildren();
+        for (var i in locations) {
+            locationMenu.add(new ox.gui.MenuItem({
+                title: noI18n(locations[i].name),
+                action: (function(id) {
+                    return function() {
+                        active.update(id);
+                        return true;
+                    };
+                })(i)
+            }));
+        }
+        active.update(active.value, join.add());
+        config.modules["com.4psa.voipnow"].gui.locations = locations;
+    });
+    
+    var tab = new ox.gui.PanelMenu("", _("VoipNow"));
+    tab.add(section);
+    return tab;
+}
+
+function callButtonAction() {
+    var collection = { objects: getContacts(), columns: phoneFields };
+    OXCache.newRequest(null, "contacts", collection, null,
+        function(data) {
+            var contact = data.objects[0];
+            var uid = contact.internal_userid;
+            if (uid) {
+                ox.JSON.put(AjaxRoot + "/user?action=list&columns=1" + 
+                    "&com.4psa.voipnow=mainExtension&session=" + session,
+                    [contact.internal_userid],
+                    function(data) {
+                        var ext = data.data[0][1];
+                        ext = ext && /^(?:\d+=)?(.*)$/.exec(ext)[1];
+                        if (ext) newcall(ext); else popup();
+                    });
+            } else {
+                popup();
+            }
+            function popup() {
+                callDialog.show(contact);
+            }
+            function newcall(number) {
+                active.wait(function(active) {
+                    ox.JSON.put(AjaxRoot +
+                        "/com.4psa.voipnow?action=newcall&phone=" +
+                        encodeURIComponent(number.replace(/[^0-9#*+]/g, "")) +
+                        "&callerid=" +
+                        encodeURIComponent(contact.display_name) +
+                        "&session=" + session, getPhones(), emptyFunction);
+                });
+            }
+        });
+}
 
 // Sending faxes
 
 var faxFields = ["fax_business", "fax_home", "fax_other"];
-MenuNodes.createSmallButton(menu, "com.4psa.voipnow.fax", _("Fax"),
-    urlify("plugins/com.4psa.voipnow/images/fax.png"),
-    urlify("plugins/com.4psa.voipnow/images/fax_d.png"),
-    function() {
-        switch (activemodule) {
-            case "contacts":
-                var collection = { objects: getContacts(), columns: faxFields };
-                OXCache.newRequest(null, "contacts", collection, null,
-                    function(data) {
-                        var numbers = [], contacts = [];
-                        for (var i = 0; i < data.objects.length; i++) {
-                            var contact = data.objects[i];
-                            for (var j = 0; j < faxFields.length; j++) {
-                                if (contact[faxFields[j]]) {
-                                    numbers.push(contact[faxFields[j]]);
-                                    contacts.push(contact);
-                                    break;
-                                }
+function faxButtonAction() {
+    switch (activemodule) {
+        case "contacts":
+            var collection = { objects: getContacts(), columns: faxFields };
+            OXCache.newRequest(null, "contacts", collection, null,
+                function(data) {
+                    var numbers = [], contacts = [];
+                    for (var i = 0; i < data.objects.length; i++) {
+                        var contact = data.objects[i];
+                        for (var j = 0; j < faxFields.length; j++) {
+                            if (contact[faxFields[j]]) {
+                                numbers.push(contact[faxFields[j]]);
+                                contacts.push(contact);
+                                break;
                             }
                         }
-                        if (contacts.length == data.objects.length) {
-                            newFax(numbers);
+                    }
+                    if (contacts.length == data.objects.length) {
+                        newFax(numbers);
+                    } else {
+                        if (contacts.length) {
+                            ox.Configuration.error(
+                                _("Not all selected contacts have an associated fax number."));
                         } else {
-                            if (contacts.length) {
-                                ox.Configuration.error(
-                                    _("Not all selected contacts have an associated fax number."));
-                            } else {
-                                ox.Configuration.error(
-                                    ngettext("The selected contact has no associated fax number.",
-                                        "The selected contacts have no associated fax number.",
-                                        data.objects.length));
-                            }
+                            ox.Configuration.error(
+                                ngettext("The selected contact has no associated fax number.",
+                                    "The selected contacts have no associated fax number.",
+                                    data.objects.length));
                         }
-                    });
-                break;
-            case "infostore":
-                var requests = new Array(oLastSelectedInfoObject.length);
-                for (var i = 0; i < oLastSelectedInfoObject.length; i++) {
-                    requests[i] = {
-                        module: "infostore", action: "get",
-                        folder: oLastSelectedInfoObject[i].folder,
-                        id: oLastSelectedInfoObject[i].id
-                    };
-                }
-                ox.JSON.put(AjaxRoot + "/multiple?session=" + session, requests,
-                    function(data) {   
-                        var items = new Array(data.length);
-                        var allFiles = true;
-                        for(var i = 0; i < data.length; data++) {
-                            var obj = data[i].data;
-                            if (obj.file_size <= 0) allFiles = false;
-                            items[i] = {
-                                filename: obj.filename,
-                                size: obj.file_size,
-                                content_type: obj.file_mimetype,
-                                id: obj.id
-                            };
-                        }
-                        if (allFiles) newFax(0, items);
-                    });
-                break;
-            default:
-                newFax();
-        }
-    });
+                    }
+                });
+            break;
+        case "infostore":
+            var requests = new Array(oLastSelectedInfoObject.length);
+            for (var i = 0; i < oLastSelectedInfoObject.length; i++) {
+                requests[i] = {
+                    module: "infostore", action: "get",
+                    folder: oLastSelectedInfoObject[i].folder,
+                    id: oLastSelectedInfoObject[i].id
+                };
+            }
+            ox.JSON.put(AjaxRoot + "/multiple?session=" + session, requests,
+                function(data) {   
+                    var items = new Array(data.length);
+                    var allFiles = true;
+                    for(var i = 0; i < data.length; data++) {
+                        var obj = data[i].data;
+                        if (obj.file_size <= 0) allFiles = false;
+                        items[i] = {
+                            filename: obj.filename,
+                            size: obj.file_size,
+                            content_type: obj.file_mimetype,
+                            id: obj.id
+                        };
+                    }
+                    if (allFiles) newFax(0, items);
+                });
+            break;
+        default:
+            newFax();
+    }
+    return true;
+}
 
 function newFax(faxes, attachments) {
-    var url = "newMail.html#to=" +
-        encodeURIComponent(configGetKey("modules")["com.4psa.voipnow"].fax);
+    var params = { to: configGetKey("modules")["com.4psa.voipnow"].fax };
     if (faxes) {
         for (var i = 0; i < faxes.length; i++) {
             faxes[i] = faxes[i].replace(/[^0-9#*+]/g, "");
         }
-        url += "&subject=" + encodeURIComponent(faxes.join(" "));
+        params.subject = faxes.join(" ");
     }
-    var w = newWindow(url, getWindowOptions());
-    if (w) {
+    ox.api.window.open("newMail.html", { params: params, success: function(w) {
+        if (ox.api.window.isEmbedded()) {
+            modify(w);
+        } else if (IE < 9) {
+            w.attachEvent("onload", cb);
+        } else {
+            addDOMEvent(w, "load", cb);
+        }
+        function cb() {
+            if (IE < 9) {
+                w.detachEvent("onload", cb);
+            } else {
+                removeDOMEvent(w, "load", cb);
+            }
+            modify(w);
+        }
+    } });
+    function modify(w) {
         if (attachments) w.aInfoMailAttachmentFile = attachments;
-        var oldInit = w.initCallback;
         function w$(id) { return w.document.getElementById(id); }
-        w.initCallback = function() {
-            w.document.title = _("Fax");
-            function rapelaceText(node, text) {
-                delete w.init.i18n[node.id];
-                node.removeChild(node.firstChild);
-                node.appendChild(addTranslated(text));
+        w.document.title = String(_("Fax"));
+        function rapelaceText(node, text) {
+            delete w.init.i18n[node.id];
+            node.removeChild(node.firstChild);
+            node.appendChild(w.newtext(text));
+        }
+        var tr = w$("subject").parentNode.parentNode;
+        rapelaceText(tr.getElementsByTagName("span")[0], String(_("To")));
+        var trs = tr.parentNode.childNodes;
+        for (var i = 0; i < trs.length; i++) {
+            if (trs[i] !== tr && trs[i].nodeType == 1) {
+                trs[i].style.display = "none";
             }
-            var tr = w$("subject").parentNode.parentNode;
-            rapelaceText(tr.getElementsByTagName("span")[0], _("To"));
-            w.changeDisplay("mail_new", "menu_new_mail", true);
-            w.changeDisplay("mail_new", "menu_new_mail_view", true);
-            var trs = tr.parentNode.childNodes;
-            for (var i = 0; i < trs.length; i++) {
-                if (trs[i] !== tr && trs[i].nodeType == 1) {
-                    trs[i].style.display = "none";
-                }
+        }
+        w.MailFunctions.prototype.showAdditionalHeaderField =
+            w.MailFunctions.prototype.drawAttachmentPanel;
+        var oldSendMessage = w.MailFunctions.prototype.sendMessage;
+        w.MailFunctions.prototype.sendMessage = function(param) {
+            if (!param) this.getMailValues();
+            if (!this.oo.subject) {
+                w.setTimeout(function() {
+                    w.triggerEvent("OX_New_Error", 4,
+                        _("Please specify at least one recipient fax number."));
+                }, 0);
+                return;
             }
-            w.MailFunctions.prototype.showAdditionalHeaderField =
-                w.MailFunctions.prototype.drawAttachmentPanel;
-            rapelaceText(w$("mail.detail.foldername"), _("Fax"));
-            var oldSendMessage = w.MailFunctions.prototype.sendMessage;
-            w.MailFunctions.prototype.sendMessage = function(param) {
-                if (!param) this.getMailValues();
-                if (!this.oo.subject) {
-                    w.setTimeout(function() {
-                        w.triggerEvent("OX_New_Error", 4,
-                            _("Please specify at least one recipient fax number."));
-                    }, 0);
-                    return;
-                }
-                var a = new w.Array();
-                a.push({ content: "", content_type: "TEXT/PLAIN" });
-                if (this.oo.attachments[0].content.length) {
-                    a.push({ content: this.oo.attachments[0].content,
-                             content_type: "TEXT/PLAIN" });
-                }
-                for (var i = 1; i < this.oo.attachments.length; i++) {
-                    a.push(this.oo.attachments[i]);
-                }
-                this.oo.attachments = a;
-                var Self = this;
-                ox.JSON.get(AjaxRoot +
-                    "/com.4psa.voipnow?action=faxauth&subject=" +
-                    encodeURIComponent(this.oo.subject) +
-                    "&session=" + session,
-                    function(reply) {
-                        Self.oo.headers = reply.data;
-                        oldSendMessage.call(Self, true);
-                    });
-            };
-            if (oldInit) oldInit.apply(this, arguments);
+            var a = new w.opener.Array();
+            a.push({ content: "", content_type: "TEXT/PLAIN" });
+            if (this.oo.attachments[0].content.length) {
+                a.push({ content: this.oo.attachments[0].content,
+                         content_type: "TEXT/PLAIN" });
+            }
+            for (var i = 1; i < this.oo.attachments.length; i++) {
+                a.push(this.oo.attachments[i]);
+            }
+            this.oo.attachments = a;
+            var Self = this;
+            ox.JSON.get(AjaxRoot +
+                "/com.4psa.voipnow?action=faxauth&subject=" +
+                encodeURIComponent(this.oo.subject) +
+                "&session=" + session,
+                function(reply) {
+                    Self.oo.headers = reply.data;
+                    oldSendMessage.call(Self, true);
+                });
+        };
+        var oldInitToolbar = w.ox.gui.initToolBarNewWindowMail;
+        w.ox.gui.initToolBarNewWindowMail = function() {
+            oldInitToolbar.apply(this, arguments);
+            var tab = w.ox.widgets.toolBar.viewControl.
+                views["new-mail-menu"].getChildren()[0];
+            tab.setTitle(_("Fax"));
+            var section = tab.getChildren()[0];
+            section.removeChild(section.getChildren()[1]);
+            tab.removeChild(tab.getChildren()[2]);
+            tab.validate();
         };
     }
 }
 
-// Call history menu entry
-MenuNodes.createSmallButton(menu, "com.4psa.voipnow.callhistory",
-    _("Call history"), "", "",
-    function() {
-        triggerEvent("OX_Switch_View", "com.4psa.voipnow.callhistory");
-    });
-
-
 // My Location
 function Updatable() {
     this.callbacks = [];
@@ -724,35 +712,6 @@ function getPhones() {
     return phones;
 }
 
-MenuNodes.createSmallButton(menu, "com.4psa.voipnow.mylocation",
-    _("My location"), "", "",
-    function() {
-        if (PopupMenu.last == locationMenu) {
-            PopupMenu.last = null;
-        } else {
-            locationMenu.show();
-        }
-    });
-var locationMenu = new ManualPulldownMenu(
-    allnodes["com.4psa.voipnow.mylocation"]);
-locations.register(function(locations, join) {
-    locationMenu.removeAllItems();
-    for (var i in locations) {
-        locationMenu.addItem(new MenuItem(noI18n(locations[i].name),
-            (function(id) { return function() { active.update(id); }; })(i)));
-    }
-    active.update(active.value, join.add());
-    config.modules["com.4psa.voipnow"].gui.locations = locations;
-});
-$("com.4psa.voipnow.mylocation").oxSubmenu = function(menu) {
-    locations.wait(function(locations) {
-        for (var i in locations) {
-            menu.addItem(new MenuItem(noI18n(locations[i].name), (function(id) {
-                return function() { active.update(id); };
-            })(i)));
-        }
-    });
-};
 var oldPhones;
 active.register(function(active, join) {
     function equals(a, b) {
@@ -811,6 +770,7 @@ function CallHistory() {
      *  The event "Update" fires when new calls are available.
      */  
     this.events = new Events();
+    this.callbacks = [];
     var Self = this;
     this.update_cb = function() { Self.update(); };
     this.update();
@@ -819,6 +779,7 @@ function CallHistory() {
 CallHistory.prototype = {
     getData: function(start, end, callback) {
         var Self = this;
+        this.updating = true;
         ox.JSON.get(AjaxRoot + "/com.4psa.voipnow?action=callreport&start=" +
             start + "&end=" + end + "&session=" + session, cb);
         function cb(reply) {
@@ -827,13 +788,15 @@ CallHistory.prototype = {
             for (var i = 0; i < calls.length; i++) {
                 var call = calls[i];
                 call.id = ++Self.id;
-                call.number = call.flow == "out" ? call.source // TODO swap
-                                                 : call.destination;
+                call.number = call.flow == "out" ? call.destination
+                                                 : call.source;
             }
             callback(calls.reverse());
         }
     },
     update: function(callback) {
+        if (callback) this.callbacks.push(callback);
+        if (this.updating) return; 
         var Self = this;
         var end = Math.floor(now() / 864e5) * 864e5; // ms/day
         this.getData(this.end, end, cb);
@@ -851,23 +814,40 @@ CallHistory.prototype = {
                 Self.end = end;
                 Self.events.trigger("Update");
             }
-            if (callback) callback();
+            for (var i = 0; i < Self.callbacks.length; i++) Self.callbacks[i]();
         }
     },
     get: function(days, callback) {
+        var Self = this;
         var start = (Math.floor(now() / 864e5) - days + 1) * 864e5; // ms/day
-        if (start < this.start) {
-            this.data = [];
-            this.start = this.end = start;
-            this.today = 0;
+        if (start >= this.start) {
+            this.update(callback);
+        } else if (this.updating) {
+            this.callbacks.push(get_cb);
+        } else {
+            get_cb();
+        }
+        function get_cb() {
+            Self.data = [];
+            Self.start = Self.end = start;
+            Self.today = 0;
+            Self.update(callback);
         }
-        this.update(callback);
     }
 };
 
 var history = new CallHistory();
 register("OX_Refresh", function() { history.update(); });
 
+// Menu
+
+menulastviews["com.4psa.voipnow.callhistory"] = "com.4psa.voipnow.callhistory";
+var menu = new ox.gui.TabMenu({ id: "com.4psa.voipnow.callhistory" });
+var filterTab = new ox.gui.PanelMenu(undefined, _("Filter"));
+menu.add(filterTab);
+ox.widgets.toolBar.viewControl.add(menu);
+menu.addListener("widget:togglehover", ox.ToolBarController.resize);
+
 // TODO filter: days
 
 function View(title, history) {
@@ -876,28 +856,46 @@ function View(title, history) {
     this.result = 1;
     var Self = this;
     
-    var s = new MenuSelection("com.4psa.voipnow", _("Status"),
-        [_("Missed"), _("Taken"), _("All")], [1, 0, 2]);
-    s.onChange = function(result) {
-        Self.result = result;
-        Self.update();
-    };
-    changeDisplay("com.4psa.voipnow.callhistory", s.id);
-    s = new MenuSelection("com.4psa.voipnow", _("Calls"),
-        [_("Incoming"), _("Outgoing"), _("All")], [0, 1, 2]);
-    s.onChange = function(flow) {
-        Self.flow = flow;
-        Self.update();
-    };
-    changeDisplay("com.4psa.voipnow.callhistory", s.id);
+    var groupID = 0;
+    function createMenuSelection(title, names, values, callback) {
+        var section = new ox.gui.PanelMenuSection(title);
+        var group = "com.4psa.voipnow." + groupID++;
+        for (var i = 0; i < names.length; i++) {
+            section.add(new ox.gui.MenuItem({
+                title: names[i],
+                behavior: "radio",
+                group: group,
+                action: (function(value) {
+                    return function() { callback(value); };
+                })(values[i])
+            }));
+        }
+        section.getChildren()[0].setChecked(true);
+        filterTab.add(section);
+    }
+    createMenuSelection(_("Status"),
+        [_("Missed"), _("Taken"), _("All")], [1, 0, 2],
+        function(result) {
+            Self.result = result;
+            Self.update();
+        });
+    createMenuSelection(_("Calls"),
+        [_("Incoming"), _("Outgoing"), _("All")], [0, 1, 2],
+        function(flow) {
+            Self.flow = flow;
+            Self.update();
+        });
     
     registerView("com.4psa.voipnow.callhistory", function() {
         showNode("modules");
         if (!Self.initialized && Self.init) Self.init();
-        ox.Configuration.View.i18n.callback = title;
+        ox.Configuration.View.i18n.callback = function() {
+            return expectI18n(title);
+        };
         ox.Configuration.View.i18n.update();
         ox.Configuration.View.i18n.enable();
         $("modules.content").appendChild(Self.content);
+        temporary.configuration.showToolbar(menu);
     }, function() {
         ox.Configuration.View.current = Self;
         Self.enter();
@@ -926,7 +924,7 @@ View.prototype = {
                 sortable: true,
                 id: "flow",
                 clear: function(div) {
-                    var src = getFullImgSrc("img/dumy.gif");
+                    var src = getFullImgSrc("img/dummy.gif");
                     if (div.firstChild) {
                         div.firstChild.src = src;
                     } else {
@@ -980,7 +978,7 @@ View.prototype = {
                     if (data <= 0) text = "";
                     //#. Abbreviation for '%d seconds'
                     //#, c-format
-                    else if (data < 60) text = format(_("%d s"), data)();
+                    else if (data < 60) text = format(_("%d s"), data);
                     else if (data < 3600) text = formatDateTime("m:ss", d);
                     else if (data < 864e5) text = formatDateTime("H:mm:ss", d);
                     else text = getInterval(d);
@@ -1045,11 +1043,11 @@ View.prototype = {
 
 var callHistory = new View(_("Call history"), history);
 
-var callHistoryImg =
-    allnodes["com.4psa.voipnow.callhistory"].firstChild.firstChild.firstChild;
 function setMissedCalls(missed) {
-    callHistoryImg.src = getFullImgSrc(missed ? "img/mail/email_priohigh.gif"
-                                              : "img/dummy.gif");
+    for (var i = 0; i < callHistoryButtons.length; i++) {
+        callHistoryButtons[i].setIcon(missed ? "img/mail/email_priohigh.gif"
+                                      : "img/dummy.gif");
+    }
 }
 
 var oldMissedCalls = 0, missedCalls = 0;
@@ -1062,55 +1060,34 @@ history.events.register("Update", function() {
     setMissedCalls(missedCalls > oldMissedCalls);
 });
 
-// Menu
-
-addMenuNode(menu.node, MenuNodes.DYNAMIC, 3);
-function updateMenu() {
-    menuglobalzaehler = 0;
-    menuarrows["com.4psa.voipnow"]["com.4psa.voipnow.menu"] = [];
-    // call
-   	menu_display_contents("com.4psa.voipnow", "com.4psa.voipnow.menu",
-   		activemodule == "contacts" ? menucountselected == 1
-   		                           : HideMenuContent,
-	    "com.4psa.voipnow.call");
-    // fax
-    menu_display_contents("com.4psa.voipnow", "com.4psa.voipnow.menu",
-        true, "com.4psa.voipnow.fax");
-    // location
-    menu_display_contents("com.4psa.voipnow", "com.4psa.voipnow.menu",
-        true, "com.4psa.voipnow.mylocation");
-    // call history
-    menu_display_contents("com.4psa.voipnow", "com.4psa.voipnow.menu",
-        true, "com.4psa.voipnow.callhistory");
-}
-var allModules = { portal: 1, mail: 1, calendar: 1, contacts: 1, tasks: 1,
-    infostore: 1, messaging: 1, "com.4psa.voipnow.callhistory": 1 };
-for (var i in allModules) changeDisplay(i, "com.4psa.voipnow.menu");
-if (currentpath2[0] in allModules) {
-    showNode("com.4psa.voipnow.menu");
-    updateMenu();
-    showArrowPopUpMenu("com.4psa.voipnow.menu.enhancement",
-                       "com.4psa.voipnow.menu", "com.4psa.voipnow");
+var allModules = ["toolbar-portal", "toolbar-mail", "toolbar-calendar",
+                  "toolbar-contacts", "toolbar-tasks", "toolbar-infostore",
+                  "toolbar-messaging", "com.4psa.voipnow.callhistory"];
+for (var i = 0; i < allModules.length; i++) {
+    var view = ox.widgets.toolBar.viewControl.views[allModules[i]];
+    if (view) view.add(createGlobalTab());
 }
 
 // Icons in mail lists
-temporary.lists.mail.addIcon({
-    name: ["headers/X-voipnow-email-type"],
-    width: "16px",
-    isVisible: function(mail) {
-        return Boolean(mail.headers && mail.headers["X-voipnow-email-type"]);
-    },
-    getIcon: function(mail) {
-        switch (mail.headers["X-voipnow-email-type"]) {
-            case "voicemail":
-                return "/plugins/com.4psa.voipnow/images/voicemail.png";
-            case "fax":
-                return "/plugins/com.4psa.voipnow/images/fax.png";
-            default:
-                return "img/dummy.gif";
+if (temporary.lists.mail) {
+    temporary.lists.mail.addIcon({
+        name: ["headers/X-voipnow-email-type"],
+        width: "16px",
+        isVisible: function(mail) {
+            return Boolean(mail.headers && mail.headers["X-voipnow-email-type"]);
+        },
+        getIcon: function(mail) {
+            switch (mail.headers["X-voipnow-email-type"]) {
+                case "voicemail":
+                    return "/plugins/com.4psa.voipnow/images/voicemail.png";
+                case "fax":
+                    return "/plugins/com.4psa.voipnow/images/fax.png";
+                default:
+                    return "img/dummy.gif";
+            }
         }
-    }
-});
+    });
+}
 
 // Configuration
 new ox.Configuration.InnerNode(
@@ -1235,44 +1212,40 @@ page.init = function() {
         });
     };
     
-    var menu = MenuNodes.createSmallButtonContext("com.4psa.voipnow.locations",
-        _("Locations"));
-    MenuNodes.createSmallButton(menu, "com.4psa.voipnow.locations.new",
-        _("New"), getFullImgSrc("img/dummy.gif"),
-        getFullImgSrc("img/dummy.gif"),
-        function() {
-            page.addNew({ name: _("New location"),
-                          phones: phones.value.phones });
-        });
-    MenuNodes.createSmallButton(menu, "com.4psa.voipnow.locations.delete",
-        _("Delete"), getFullImgSrc("img/menu/delete.gif"),
-        getFullImgSrc("img/menu/delete_d.gif"), confirmDelete);
+    page.toolbar = temporary.configuration.newToolbar(_("Locations"),
+        [temporary.configuration.saveButton, {
+            title: _("Locations"),
+            buttons: [{
+                title: _("New"),
+                action: function() {
+                    page.addNew({ name: _("New location"),
+                                  phones: phones.value.phones });
+                }
+            }, {
+                title: _("Delete"),
+                icons: ["img/menu/delete.gif", "img/menu/delete_d.gif"],
+                action: confirmDelete
+            }]
+        }]);
+    var deleteButton = page.toolbar.getChildren()[0].getChildren()[1].
+        getChildren()[1];
     function confirmDelete() {
         var count = page.list.selection.count;
         if (!count) return;
         newConfirm(
-            ngettext("Delete location", "Delete locations", count)(),
+            ngettext("Delete location", "Delete locations", count),
             ngettext("Are you sure you want to delete the selected location?",
                 "Are you sure you want to delete the selected locations?",
-                count)(),
+                count),
             AlertPopup.YES | AlertPopup.NO, null, null, handleDelete);
     }
     function handleDelete() {
         page.list.deleteIDs(page.list.selection.getSelected());
         saveLocations(emptyFunction);
     }
-    addMenuNode(menu.node, MenuNodes.FIXED, 31);
-    changeDisplay(node.id, "com.4psa.voipnow.locations");
-    menuarrows[node.id] = {};
     register("OX_SELECTED_ITEMS_CHANGED", function() {
-        menuglobalzaehler = 0;
-        menuarrows[node.id]["com.4psa.voipnow.locations"] = [];
-        menu_display_contents(node.id, "com.4psa.voipnow.locations",
-            true, "com.4psa.voipnow.locations.new");
         var count = page.list.selection.count;
-        menu_display_contents(node.id, "com.4psa.voipnow.locations",
-            count > 0 && storage.ids.length > count,
-            "com.4psa.voipnow.locations.delete");
+        deleteButton.setEnabled(count > 0 && storage.ids.length > count);
     });
     
 };
\ No newline at end of file
